
#ifndef DEVMEMGPIOHINCLUDED
#define DEVMEMGPIOHINCLUDED 1

#include <stdint.h>

/* Look at /proc/cpuinfo and return the revision.
    Revision 0010 is B+, 
    0008 is model A, 
    000e is UK made model B2
    000f is Chinese made model B2
    and so on... 
*/ 
int findrev();

/* Model Pi 2 has a different GPIO base than the earlier models */

/* #define BCM2708_PERI_BASE 0x20000000 Pi A, A+, B, B+ 256MB, 512 MB */ 
/* #define BCM2708_PERI_BASE 0x3F000000 Pi 2 1GB */ 
/* Somehow this translates to 0x7E00 0000  */ 

/* The ModelToPERIBase(modelno) returns the appropriate address. */

/* Model Pi 2 has a different periferal base than the earlier models */
uint32_t ModelToPERIBase(int fdrv);

/* GPIO controller */
uint32_t GPIO_BASE(uint32_t peri_base);

/* PWM controller */
uint32_t PWM_BASE(uint32_t peri_base);

/* Clock controller */
uint32_t CLOCK_BASE(uint32_t peri_base);



/* Uses the /dev/mem addresses to change GPIO lines. 
    Based on code seen in 

    http://elinux.org/RPi_Low-level_peripherals

    We can access gpios in device drivers as well, but then we have
    to re-do the kernel; we may not always want to have to do that.
*/

#if(0)

#define BLOCK_SIZE (4*1024)
#define PAGE_SIZE (4*1024)

// GPIO setup macros. Always use INP_GPIO(x) before using OUT_GPIO(x) or SET_GPIO_ALT(x,y)
#define INP_GPIO(g) *(gpio+((g)/10)) &= ~(7<<(((g)%10)*3))
#define OUT_GPIO(g) *(gpio+((g)/10)) |=  (1<<(((g)%10)*3))

/* Documentation: register-addresses for 
    Output Set at +1C +20 
    Output Clear +28 +2C
    Input Level at +34 +38
*/

#define SET_GPIO_ALT(g,a) *(gpio+(((g)/10))) |= (((a)<=3?(a)+4:(a)==4?3:2)<<(((g)%10)*3))

#define GPIO_SET *(gpio+7)  // sets   bits which are 1 ignores bits which are 0 register +1C (decimal 28, 7*4)
#define GPIO_CLR *(gpio+10) // clears bits which are 1 ignores bits which are 0 register +28 (decimal 40, 7*10)

#endif


#if(0)


/* Argument: allow us to show build version (date)
    as well as limiting the list of output. */ 
int main(int argc, char **argv)
{
    int mem_fd;
    volatile uint32_t *gpio;
    char *gpio_map;
    int fdrv;
    char gstyl[20];
    char minfo[30];
    uint32_t gpio0;
    int j;
    int devtypeonly;

    devtypeonly = 0;

    /* Look at the arguments. */ 
    for(j = 1; j<argc; j++)
    {
        if(!strcmp("-?", argv[j]))
		{
			printf("usage:\n"
				"gpiostates -v -d\n"
				"where:\n"
				"-v show version\n"
				"-d stop after identifying this machine\n"
				"-? this list\n"
				); 
			return(0);
		}

        if(!strcmp("-v", argv[j]))
        {
            printf("gpiostates %s %s \n", __DATE__, __TIME__); 
        }

        if(!strcmp("-d", argv[j]))
        {
            devtypeonly = 1;
        }
    }

    fdrv = findrev();

    switch(fdrv)
    {
    /* From elinux.org/RPi_HardwareHistory */ 

    case 0x02:
        snprintf(gstyl, 20, "%s", "G26" );
        snprintf(minfo, 30, "%s", "Model B R1 256M" );
        break;

    case 0x03:
        snprintf(gstyl, 20, "%s", "G26" );
        snprintf(minfo, 30, "%s", "Model B R1 256M (ECN0001)" );
        break;

    case 0x04:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model B R2 256M (Sony)" );
        break;
    case 0x05:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model B R2 256M (Qisda)" );
        break;
    case 0x06:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model B R2 256M (Egoman)" );
        break;
    case 0x07:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model A (Egoman)" );
        break;
    case 0x08:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model A (Sony)" );
        break;
    case 0x09:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model A (Qisda)" );
        break;

    case 0x0d:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model B2 512M (Egoman)" );
        break;

    case 0x0e:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model 2 512MB (Sony)" );
        break;

    case 0x0f:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Model 2 512MB (Qisda)" );
        break;

    case 0x10:
        snprintf(gstyl, 20, "%s", "G40" );
        snprintf(minfo, 30, "%s", "Model B+" );
        break;

    case 0x11:
        snprintf(gstyl, 20, "%s", "G40" ); // Details TBD! 
        snprintf(minfo, 30, "%s", "Compute Module" );
        break;

    case 0x12:
        snprintf(gstyl, 20, "%s", "G40" );
        snprintf(minfo, 30, "%s", "Model A+" );
        break;

    case 0xa21041:
        snprintf(gstyl, 20, "%s", "G40" );
        snprintf(minfo, 30, "%s", "Model B2 1GB (China)" );
        break;

    case 0xa01041:
        snprintf(gstyl, 20, "%s", "G40" );
        snprintf(minfo, 30, "%s", "Model B2 1GB (UK)" );
        break;

    default:
        snprintf(gstyl, 20, "%s", "G26+8" );
        snprintf(minfo, 30, "%s", "Unrecognized" );
        break;
    }

    printf("Found revision = 0x%x (%s) %s\n", fdrv, gstyl, minfo); 

    if(devtypeonly)
    {
        return(0);
    }

    gpio0 = ModelToGPIOBase(fdrv);

    mem_fd = open("/dev/mem", O_RDWR | O_SYNC);
    if(mem_fd < 0)
    {
        perror("/dev/mem");
        exit(2);
    }
    
    gpio_map = (char *) mmap(NULL, 
        BLOCK_SIZE, 
        PROT_READ|PROT_WRITE,
        MAP_SHARED,
        mem_fd,
        gpio0); 

    close(mem_fd);

    gpio = (volatile uint32_t *) gpio_map;
    showfsel(gpio, gstyl); 
    return(0);
}

#endif
#endif

